/*
 * Simple alternative GstMixer implementation with ALSA-native API
 */

#ifndef __XFCE4_MIXER_ALSA_H
#define __XFCE4_MIXER_ALSA_H

G_BEGIN_DECLS

/*
 * GstMixer
 */

GType gst_mixer_get_type (void);

#define GST_TYPE_MIXER \
  (gst_mixer_get_type ())
#define GST_MIXER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_MIXER, GstMixer))
#define GST_MIXER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_MIXER, GstMixerClass))
#define GST_IS_MIXER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_MIXER))
#define GST_IS_MIXER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_MIXER))
#define GST_MIXER_GET_CLASS(inst) \
  (G_TYPE_INSTANCE_GET_INTERFACE ((inst), GST_TYPE_MIXER, GstMixerClass))

typedef struct _GstMixer GstMixer;
typedef struct _GstMixerClass GstMixerClass;

typedef enum {
  GST_MIXER_FLAG_NONE                = 0,
  GST_MIXER_FLAG_AUTO_NOTIFICATIONS  = (1<<0),
  GST_MIXER_FLAG_HAS_WHITELIST       = (1<<1),
  GST_MIXER_FLAG_GROUPING            = (1<<2),
} GstMixerFlags;

typedef enum {
  GST_MIXER_MESSAGE_INVALID,
  GST_MIXER_MESSAGE_MUTE_TOGGLED,
  GST_MIXER_MESSAGE_RECORD_TOGGLED,
  GST_MIXER_MESSAGE_VOLUME_CHANGED,
  GST_MIXER_MESSAGE_OPTION_CHANGED,
  GST_MIXER_MESSAGE_OPTIONS_LIST_CHANGED,
  GST_MIXER_MESSAGE_MIXER_CHANGED
} GstMixerMessageType;

struct _GstMixer {
  GstElement element;
  GList *tracklist;
  void *handle; /* snd_mixer_t */
  const char *name;
  const gchar *card_name;
  GSource *src;
};

struct _GstMixerClass {
  GstElementClass parent_class;
};

const GList *gst_mixer_list_tracks (GstMixer *mixer);

static inline GstMixerFlags
gst_mixer_get_mixer_flags (GstMixer * mixer)
{
  return GST_MIXER_FLAG_AUTO_NOTIFICATIONS;
}

GstMixerMessageType gst_mixer_message_get_type (GstMessage *message);

int gst_mixer_new (const char *name, GstMixer **mixer_ret);
GList *gst_mixer_probe_devices (void);
const gchar *gst_mixer_get_card_name (GstMixer *mixer);

/*
 * GstMixerTrack
 */

GType gst_mixer_track_get_type (void);

#define GST_TYPE_MIXER_TRACK \
  (gst_mixer_track_get_type ())
#define GST_MIXER_TRACK(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_MIXER_TRACK, \
                               GstMixerTrack))
#define GST_MIXER_TRACK_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_MIXER_TRACK, \
                            GstMixerTrackClass))
#define GST_IS_MIXER_TRACK(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_MIXER_TRACK))
#define GST_IS_MIXER_TRACK_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_MIXER_TRACK))

typedef struct _GstMixerTrack GstMixerTrack;
typedef struct _GstMixerTrackClass GstMixerTrackClass;

typedef enum {
  GST_MIXER_TRACK_INPUT  = (1<<0),
  GST_MIXER_TRACK_OUTPUT = (1<<1),
  GST_MIXER_TRACK_MUTE   = (1<<2),
  GST_MIXER_TRACK_RECORD = (1<<3),
  GST_MIXER_TRACK_MASTER = (1<<4),
  GST_MIXER_TRACK_SOFTWARE = (1<<5),
  GST_MIXER_TRACK_NO_RECORD = (1<<6),
  GST_MIXER_TRACK_NO_MUTE = (1<<7),
  GST_MIXER_TRACK_WHITELIST = (1<<8),
  GST_MIXER_TRACK_READONLY = (1<<9),
  GST_MIXER_TRACK_WRITEONLY = (1<<10)
} GstMixerTrackFlags;

struct _GstMixerTrack {
  GObject parent;
  void *element;
  gchar *label;
  gchar *untranslated_label;
  guint index;
  GstMixerTrackFlags flags;
  gint num_channels;
  gint *volumes;
  gint min_volume;
  gint max_volume;
  GstMixerTrack *shared_mute;
  gboolean has_volume;
  gboolean has_switch;
};

struct _GstMixerTrackClass {
  GObjectClass parent;
};

#define GST_MIXER_TRACK_HAS_FLAG(track, flag)	((track)->flags & (flag))

void gst_mixer_get_volume (GstMixer *mixer, GstMixerTrack *track, gint *volumes);
void gst_mixer_set_volume (GstMixer *mixer, GstMixerTrack *track, gint *volumes);
void gst_mixer_set_mute (GstMixer *mixer, GstMixerTrack *track, gboolean mute);
void gst_mixer_set_record (GstMixer *mixer, GstMixerTrack *track, gboolean record);

void gst_mixer_message_parse_mute_toggled (GstMessage *message,
					   GstMixerTrack **track,
					   gboolean *mute);
void gst_mixer_message_parse_record_toggled (GstMessage *message,
					     GstMixerTrack **track,
					     gboolean *record);
void gst_mixer_message_parse_volume_changed (GstMessage *message,
					     GstMixerTrack **track,
					     gint **volumes,
					     gint *num_channels);

/*
 * GstMixerOptions
 */

GType gst_mixer_options_get_type (void);

#define GST_TYPE_MIXER_OPTIONS \
  (gst_mixer_options_get_type ())
#define GST_MIXER_OPTIONS(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_MIXER_OPTIONS, GstMixerOptions))
#define GST_MIXER_OPTIONS_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_MIXER_OPTIONS, GstMixerOptionsClass))
#define GST_IS_MIXER_OPTIONS(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_MIXER_OPTIONS))
#define GST_IS_MIXER_OPTIONS_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_MIXER_OPTIONS))
#define GST_MIXER_OPTIONS_GET_CLASS(inst) \
  (G_TYPE_INSTANCE_GET_INTERFACE ((inst), GST_TYPE_MIXER_OPTIONS, GstMixerOptionsClass))

typedef struct _GstMixerOptions GstMixerOptions;
typedef struct _GstMixerOptionsClass GstMixerOptionsClass;

struct _GstMixerOptions {
  GstMixerTrack parent;
  GList *values;
};

struct _GstMixerOptionsClass {
  GstMixerTrackClass parent;
};

void gst_mixer_set_option (GstMixer * mixer, GstMixerOptions * opts, gchar * value);
const gchar * gst_mixer_get_option (GstMixer * mixer, GstMixerOptions * opts);
GList * gst_mixer_options_get_values (GstMixerOptions *mixer_options);
void gst_mixer_message_parse_option_changed (GstMessage *message,
					     GstMixerOptions ** options,
					     const gchar **value);
void gst_mixer_message_parse_options_list_changed (GstMessage *message,
						   GstMixerOptions **options);

G_END_DECLS

#endif /* __XFCE4_MIXER_ALSA_H */
